/* Source by Dave Watson --- TenorDave@hotmail.com
This file is a list of functions for using the 89/92+'s link port to access a I2C device.   I have not tested this personaly, although it should work.   You will need to modify the main, and possibly the read functions to make them work with your chip.   If you find a bug or error, please report it to the above address.   The source is currently set up so it can be used with an I2C tempureture sensor, the one linked to by ticalc's hardware page.   Thanks for the support! */ 

// C Source File
// Created 6/10/2000; 11:00:44 AM

// #define USE_GRAY_PATCH
// #define USE_LONGMUL_PATCH
// #define USE_LONGDIV_PATCH
   #define USE_INITS_PATCH

#define ESC_ROW 0xffbf
#define ESC_KEY 1

#define SIZE 2

#define SAVE_SCREEN

#include <nostub.h>  // Make NoStub Program
#include <all.h>     // Include All Header Files

char n=0,BUFFER=0,x=0,*point=(char *)0x60000C,*point2=(char *)0x60000E;
char DATA[SIZE]={0};

void I2C_INIT(void);
void START(void);
char STOP(void);
void SDA(char state);
void SCK(char state);
char read_SDA(void);
void PUTBYTE(char BUFFER);
void GETBYTE(void);
void GIVEACK(void);
char GETACK(void);
char READ(char address,char Number_of_bytes);
void WRITE(char address,char Number_of_bytes);
void reset(void);
char read_SCK(void);

int _ti89;           // Produce .89Z File


int _main()
{
	char extra[2]="5";
	*point=0x40;
	
	poke(0x600017,0xFA);
	
	OSFreeTimer(6);
	OSRegisterTimer(6,1);

	ClrScr();
	I2C_INIT();
	while(!(_rowread(ESC_ROW)&ESC_KEY))
	{
		if(!READ(0x90,2))
		{
			printf_xy(0,10,"Read Failed!");
			GKeyFlush();
			ngetchx();
			reset();
			return 0;	
		}
		//DATA[0]-=5;
		if(DATA[1])
			strcpy (extra,"5"); 
		else
		  strcpy (extra,"0"); 
		printf_xy(0,20,"Temp Celcius = %i.%s",DATA[0],extra);
		printf_xy(0,30,"Temp Ferenheight = %i",DATA[0]*(9/5)+32);
	}
	ngetchx();
	
	reset();
	return 0;
}

void reset(void)
{
	*point=0x50;
	*point=0x8D;
	poke(0x600017,0xB2);
}

char read_SDA(void)
{
	return !((*point2&0x04)>>2);
}

char read_SCK(void)
{
	return !((*point2&0x08)>>3);
}

void SDA(char state)
{
	if(state==1)
		*point2&=0xFE;
	else
		*point2|=0x01;
}

void SCK(char state)
{
	if(state==1)
		*point2&=0xFD;
	else
		*point2|=0x02;
}

void I2C_INIT(void) // call this immediately after power-on
{
	SDA(1); 
	SCK(0); 
	for(n=0;n<4;n++)
	{ 
		if(STOP()==0)
		 printf_xy(0,(10*n)+40,"Stuck Bus!");
	}
}

void START(void)
{
	SCK(1); // BUGFIX ! 
	SDA(1); // Improvement  
	SDA(0); 
	SCK(0); 
	SDA(1); 
	OSTimerRestart(6);
	while(!OSTimerExpired(6));
}

char STOP(void)
{
	SDA(0); 
	SCK(1); 
	SDA(1);
	OSTimerRestart(6);
	while(!OSTimerExpired(6));	
	if((read_SDA()==0)||(read_SCK()==0))
	 	return(0);
	return(1);
}

void PUTBYTE(char BUFFER)
{ 
	for (n=7;n>-1;n--)
	{ 
		SDA((BUFFER>>n)&0x01);
		SCK(1); 
		SCK(0); 
		OSTimerRestart(6);
	  while(!OSTimerExpired(6));
	  OSTimerRestart(6);
	  while(!OSTimerExpired(6));
	}
	SDA(1); 
}

void GETBYTE(void)
{ 
	BUFFER=0;
	for (n=7;n>-1;n--)
	{ 
		SCK(1); 
		BUFFER |= read_SDA()<<n; 
		SCK(0); 
		OSTimerRestart(6);
	  while(!OSTimerExpired(6));
	} 
	SDA(1); 
}
void GIVEACK(void)
{ 
	SDA(0); 
	SCK(1); 
	SCK(0); 
	SDA(1); 
	OSTimerRestart(6);
	  while(!OSTimerExpired(6));
	OSTimerRestart(6);
	while(!OSTimerExpired(6));
}

char GETACK(void)
{ 
	char x=0;
	SDA(1); 
	SCK(1); 
	OSTimerRestart(6);
	while(!OSTimerExpired(6));
	printf_xy(0,0,"Reading...");
	GKeyFlush();
	while((read_SDA()!=0)&&(x<127))
	{
		x++;
		if(_rowread(ESC_ROW)&ESC_KEY)
			return(0);
	} 
	SCK(0);
	return(1); 
}

/* this concludes the low-level set of instructions for the I2C driver 
The next functions will handle the telegram formatting on a higher level */

char READ(char address,char Number_of_bytes)
{ 
	char Device_adress;

	Device_adress=address|0x01; //This sets the READ FLAG 
	START(); 
	PUTBYTE(Device_adress); 
	if(!GETACK())
	{
		GKeyFlush ();
		return(0);
	}
	for(x=0;x<Number_of_bytes;x++)
	{
		GETBYTE(); 
		DATA[x]=BUFFER; //Copy received BYTE to DATA array  
		if (x<Number_of_bytes-1) //Not ack the last byte
			GIVEACK();
	}
	STOP();
	return(1);
}

void WRITE(char address,char Number_of_bytes)
{ 
	char Device_adress;
	
	Device_adress=address&0xFE; // This clears READ flag
	START(); 
	PUTBYTE(Device_adress); 
	assert (GETACK()!=1);
	for(x=0;x<Number_of_bytes;x++)
	{ 
		PUTBYTE (DATA[x]);
		assert (GETACK()!=1);
	}
	STOP();
}
